{xDEFINE TEST_GAUSSIANA}
unit uanalisisserial;

interface

uses
{$IFDEF FPC-LCL}
  LResources,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  xmatdefs,
  fddp,
  matreal, matent,
  //  uhermitaprox,
  uregresion,
  StdCtrls, udisnormcan,
  uAuxiliares,
  uverdoc,
  umodelosintcegh,
  WRFFTI01, WRFFTF01,
  ExtCtrls, uhurst, autoco;

type

  { TForm1 }

  TForm1 = class(TForm)
    BAnalizar: TButton;
    eOrdenESHY: TEdit;
    ESHY: TButton;
    cbFiltroCiclico: TCheckBox;
    eOrdenDelFiltro: TEdit;
    OpenDialog1: TOpenDialog;
    LOrdenFiltro: TLabel;
    Label2: TLabel;
    eOverlapping: TEdit;
    Label3: TLabel;
    LArchiSeries: TLabel;
    eArchiSeriesDeDatos: TEdit;
    BBuscar: TButton;
    eUmbralFiltro: TEdit;
    cbFiltrarMenores: TCheckBox;
    eHorasEntreMuestras: TEdit;
    Label1: TLabel;
    GroupBox1: TGroupBox;
    BCalcularPromediosMensuales: TButton;
    BTestVarianza: TButton;
    BPredictorNino34: TButton;
    TestDistribucionGaussiana: TButton;
    GenerarRuidoRectificado: TButton;
    BLeerDatosBasicos: TButton;
    Button1: TButton;
    Label4: TLabel;
    eVarianzaDeLoNoExplicado: TEdit;
    rbg_calculoB: TRadioGroup;
    lstAniosAnalogos: TMemo;
    Label5: TLabel;
    cbAniosAnalogos: TCheckBox;
    eSemanaInicioVentanaAnaloga: TEdit;
    Label6: TLabel;
    Label7: TLabel;
    eTraslapping: TEdit;
    Label8: TLabel;
    eNPuntosPorMiniCiclo: TEdit;
    Label9: TLabel;
    eNDigitosDesformadores: TEdit;
    Label10: TLabel;
    eNDecimalesDesformadores: TEdit;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;


    procedure BAnalizarClick(Sender: TObject);
    procedure ESHYClick(Sender: TObject);
    procedure TestDistribucionGaussianaClick(Sender: TObject);
    procedure GenerarRuidoRectificadoClick(Sender: TObject);
    procedure BBuscarClick(Sender: TObject);
    procedure BLeerDatosBasicosClick(Sender: TObject);
    procedure BPredictorNino34Click(Sender: TObject);
    procedure BTestVarianzaClick(Sender: TObject);
    procedure BCalcularPromediosMensualesClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);

  private
    seriesDeDatos: TSeriesDeDatos;
    //seriesDeDatos1:  TSeriesDeDatos;
    aniosanalogos: array of integer;
    semanaDeInicio: integer;
    kBuscarDesde: integer;

  public
    { Public declarations }

    procedure GetAniosAnalogos;
    procedure MascararAniosNoAnalogos(datos: TSeriesDeDatos; mascara: double);
    function Mascarado(anio, semana: integer): boolean;
    function buscarAnio(anio: integer): integer;

  end;

var
  Form1: TForm1;

const
  UMBRAL_TOCADOS = -14.14E14; // un numero muy negativo

implementation


// recorre las series y quita los tramos en que halla actuado el filtro
// acortando las series.
procedure AcortarTramosFiltrados(var series: TDAOfVectR; umbralFiltro: NReal);

var
  j, jval, NSems: integer;
  flg_Filtrado: boolean;
  iserie: integer;

begin
  NSems := Series[0].n;
  j := 1;
  while (j <= nSems) do
  begin
    //detecto si actuó el filtro en alguna de las series.
    flg_Filtrado := False;
    for iserie := 0 to high(series) do
      if series[iserie].e(j) <= umbralFiltro then
      begin
        flg_Filtrado := True;
        break;
      end;

    // si actuó el filtro acortamos todas las series.
    if flg_Filtrado then
    begin
      for iserie := 0 to high(series) do
      begin
        for jval := j to nSems - 1 do
          series[iserie].pon_e(jval, series[iserie].e(jval + 1));
      end;
      Dec(nSems);
    end
    else
      Inc(j);
  end;

  // si corresponde acortamos las series.
  if nSems < series[0].n then
  begin
    for iserie := 0 to high(series) do
    begin
      setlength(series[iserie].pv, nSems + 1);
      series[iserie].n := nSems;
    end;
  end;
end;


// retorna la matriz B triangular Inferior de la factorización de
// Cholesky que tal que B*transpuesta(B) es la matriz de covarianzas
// de las series.
function descuadrador_Cholesky(var series: TDAOfVectR; filtrar: boolean;
  umbralFiltro: NReal): TMatR;

var
  A: TMatR; // matriz de covarianzas
  B: TMatR;
  k, j: integer;
  m: NReal;
  NDatos: integer;

begin

  // SI la orden es filtrar, acortamos las series
  // quitando los tramos en los que actuó el filtro.
  if Filtrar then
    AcortarTramosFiltrados(series, umbralFiltro);


  NDatos := series[0].n;

  A := TMatR.Create_Init(length(series), length(series));
  // construimos la matriz de covarianzas
  // como es simétrica lleno solo el triangulo superior
  for k := 1 to A.nf do
    for j := k to A.nc do
    begin
      // se supone que las series tienen valor esperado NULO
      m := series[k - 1].PEV(series[j - 1]) / NDatos;
      A.pon_e(k, j, m);
    end;

  B := A.Raiz_Cholesky;
  A.Free;
  Result := B;
end;

function descuadrador(var series: TDAOfVectR; filtrar: boolean;
  umbralFiltro: NReal): TMatR;
const
  MaxNIters = 1000;

var
  NSems, NSeries, NDescuadres: integer;

var
  x_series, x_autovects: TVectR;
  xs_series, xs_autovects: TVectR;
  autovects: TDAOfVectR;
  autovals: array of double;
  convergio: array of boolean;
  kvect: integer;
  NAutovects: integer;
  resB: TMatR;

var
  sal{, f}: textfile;
  //  r: string;
  //  ksem: integer;
  //  ap_b,ap_p,ap_s: double;

  k, j: integer;
  me2: double;

  buscando: boolean;

  descuadrando: boolean;
  niter: integer;
  factorCambioNorma: NReal;

  iserie: integer;

  varianzas: TVectR;

  sb2: NReal;

begin
  (*** OJO NO USE ESTE METODO ***)


  // SI la orden es filtrar, acortamos las series
  // quitando los tramos en los que actuó el filtro.
  if Filtrar then
    AcortarTramosFiltrados(series, umbralFiltro);

  // más vale hacer esto después de acortar las series.
  NSeries := length(Series);
  NSems := Series[0].n;

  // calculamos las varianzas
  varianzas := TVectR.Create_Init(NSeries);
  for iserie := 1 to NSeries do
    varianzas.pv[iserie] := series[iserie - 1].varianza;

  NDescuadres := NSeries;

  setlength(autovects, NDescuadres);
  setlength(autovals, NDescuadres);
  setlength(convergio, NDescuadres);


  for k := 1 to NDescuadres do
    autovects[k - 1] := TVectR.Create_Init(NSems);

  x_series := TVectR.Create_Init(NSeries);
  x_autovects := TVectR.Create_Init(NDescuadres);
  xs_series := TVectR.Create_Init(NSeries);
  xs_autovects := TVectR.Create_Init(NDescuadres);

  kvect := 1;
  descuadrando := True;

  NAutovects := NDescuadres;

  while (kvect <= NDescuadres) and (descuadrando) do
  begin
    // cargamos un versor cualquiera como candidato
    // a ser el autovalor principal del conjunto de series.
    // elegimos la serie como inicio de iteración, pero no es imperativo.
    autovects[kvect - 1].Copy(Series[kvect - 1]);
    autovects[kvect - 1].hacerunitario;

    // inicializamos variables auxiliares de la iteración
    x_series.Ceros;
    x_autovects.Ceros;
    buscando := True;
    niter := 1;


    while buscando and (niter <= MaxNIters) do
    begin
      // proyecciones sobre las series
      for k := 1 to NSeries do
        xs_series.pon_e(k, series[k - 1].PEV(autovects[kvect - 1]));
      // proyecciones sobre autovectores ya estimados
      for k := 1 to kvect - 1 do
        xs_autovects.pon_e(k, autovects[k - 1].PEV(autovects[kvect - 1]));

      // calculo de variación para dar fin a la iteración
      me2 := (xs_series.distancia2(x_series) + xs_autovects.distancia2(
        x_autovects)) / NSems;
      if (me2 < 1e-14) then
        buscando := False;
      vswap(xs_series, x_series);
      vswap(xs_autovects, x_autovects);
      // formación del vector
      autovects[kvect - 1].ceros;
      for k := 1 to NSeries do
        autovects[kvect - 1].sumRPV(x_series.e(k), series[k - 1]);

      for k := 1 to kvect - 1 do
        autovects[kvect - 1].sumRPV(-autovals[k - 1] * x_autovects.e(k),
          autovects[k - 1]);

      // usamos la raiz de la varianza como distancia
      autovals[kvect - 1] := autovects[kvect - 1].normEuclid;
      // lo hacemos unitario
      autovects[kvect - 1].PorReal(1 / autovals[kvect - 1]);
      Inc(niter);
    end;
    if buscando then
    begin
      descuadrando := False;
      NAutovects := kvect - 1;
    end
    else
    begin
      writeln('me2: ', me2, ' landa: ', autovals[kvect - 1]);
      writeln('niter: ', niter);
      Inc(kvect);
    end;
  end;


(* Escalamos los autovectores para que sean de varianza 1
en lugar de Norma Euclidea 1 *)
  factorCambioNorma := sqrt(NSems);
  for j := 1 to NAutovects do
    autovects[j - 1].PorReal(factorCambioNorma);

(* generalmente NAutovects = NSeries, sería demasiada casualidad
que las muestras dieran con un autovalor NULO *)
  resB := TMatR.Create_Init(NSeries, NAutovects);
  for k := 1 to NSeries do
    for j := 1 to NAutovects do
      resB.pon_e(k, j, series[k - 1].PEV(autovects[j - 1]) / NSems);
(*
 las filas de B son las cordenadas de las series en la base de los
 autovectores.*)
  Result := resB;

  for k := 1 to NSeries do
  begin
    sb2 := resB.Fila(k).ne2;
    resB.Fila(k).PorReal(sqrt(varianzas.e(k) / sb2));
  end;

  (*** IMPRIMIMOS el archivo de control de resultados ****)
  assignfile(sal, 'descuadre.xlt');
  rewrite(sal);
  writeln(sal, 'MATRIZ B ');
  for k := 1 to NSeries do
  begin
    Write(sal, 'serie: ', #9, k);
    for j := 1 to NAutovects do
      Write(sal, #9, resB.e(k, j));
    writelN(sal);
  end;

  writeln(sal);
  Write(sal, 'autovalores:', #9);
  for j := 1 to NAutovects do
    Write(sal, #9, autovals[j - 1]);
  writeln(sal);


  writeln(sal, ' SERIES y AUTOVECTORES ');
  for j := 1 to NSeries do
    Write(sal, #9, 's', j);
  for j := 1 to NAutovects do
    Write(sal, #9, 'v', j);
  writeln(sal);

  for k := 1 to NSems do
  begin
    Write(sal, k);
    for j := 1 to NSeries do
      Write(sal, #9, series[j - 1].e(k));
    for j := 1 to NAutovects do
      Write(sal, #9, autovects[j - 1].e(k));
    writeln(sal);
  end;
  closefile(sal);

end;



(*+doc
Esta función recibe como parámetros una serie de datos y la cantidad de puntos
de la serie que forman un ciclo.
La cantidad de puntos de un ciclo, define ciclos dentro de la serie. Los primeros
NPPorCiclo puntos de la serie pertenecen al primer ciclo, los segundos NPPorCiclo
puntos de la serie pertenecen al segundo ciclo y así sucesivamente.
Dentro de cada ciclo, los puntos se pueden indexar de 1 a NPPorciclo y diremos
que el punto es el primer paso del ciclo, el segundo y así hasta el NPPorCiclo paso
del ciclo. Todo punto de la serie entonces se clasifica dentro de un ciclo y dentro
de ese ciclo le corresponde un paso.
La idea es agrupar todos los puntos, de diferentes ciclos que tienen
el mismo paso y con ellos crear una fuente aleatoria capaz de generar números
con igual histograma que el grupo de puntos de la serie de datos que corresponden
a ese paso.
El resultado es entonces NPPorCiclo fuentes aleatorias que tienen
igual función de densidad de probabilidad que la serie original para cada punto
de un ciclo.

Si overlaping= 0 cada muestra aporta solamente al uniformizador del paso que
le corresonde dentro del sitio. Si overlapping es 1 cada muestra aporta a su
propio paso al anterior y al siguiente.

Para explicarlo en palabras, en el caso de los aportes hidráulicos, si ponemos
overlapping= 0 y poniendo NPPorCiclo=52, se crean 52 funciones uniformizantes
una para cada semana del año considerando solamente los aportes de esa semana.
Por ejemplo, para la primer semana del año, se consideran las muestras solamente
de la primer semana de cada año en la construcción de la función. Como la cantidad
de años de aportes es del orden de 100, el usar estrictamente el aporte de cada
semana en la construcción de las funciones desformantes, puede introducir un
comportamiento adicional basado en los datos. Parece más adecuado indicar que
las características de la primer semana en realidad es compartida con las semanas
de su entorno y por lo tanto se podrían usar las muestras de una semana y las
de su entorno. Poniendo overlapping=1 estaremos considerando como entorno
la semana anterior y la siguiente. Con overlapping=2 consideramos las dos
anteriores y las dos siguientes.
-doc*)
function uniformizadores(serie: TVectR; NPPorCiclo: integer;
  overlapping: integer; traslapping: integer; nPuntosPorMiniciclo: integer;
  filtrar: boolean; umbralFiltro: NReal;
  NPuntosResultado: integer): TDAOf_ddp_VectDeMuestras;

var
  kPaso, kMuestra: integer;
  nCiclos: integer;
  VectorDeMuestras: array of TVectR;
  res: TDAOf_ddp_VectDeMuestras;
  jCiclo, jOverlap, jTraslap: integer;
  vaux, va: TVectR;
  valor: NReal;
  nNoFiltrados: integer;
  cnt_Filtrados: integer;
  k: integer;
  ipos: integer;


  jrd: NReal;

begin
  // Calculamos la cantidad de ciclos enteros que caben en la serie
  nCiclos := Serie.n div NPPorCiclo;

  // Vamos a hacer un Vector de Muestras para cada paso dentro de un ciclo
  setlength(VectorDeMuestras, NPPorCiclo);

  // Vector auxiliar para soporte del conjunto ampliado de muestras
  vaux := TVectR.Create_Init(nCiclos * (1 + 2 * overlapping) * (1 + 2 * traslapping));

  for kPaso := 1 to NPPorCiclo do
  begin
    for k := 1 to vaux.n do
      vaux.pv[k] := UMBRAL_TOCADOS; // esto es para detectar puntos NO TOCADOS


    // Bien, ahora construimos los Vectores de Muestras agregando las muestras de la
    // serie que corresponden al paso de tiempo dentro del ciclo que asociamos
    // a cada vector.
    ipos := 1;
    for jCiclo := 1 to nCiclos do
      for jTraslap := -Traslapping to Traslapping do
        for jOverlap := -Overlapping to Overlapping do
        begin
          kMuestra := kPaso + (jCiclo - 1) * NPPorCiclo + jTraslap *
            NPuntosPorMiniciclo + jOverlap;
          while kMuestra < 1 do
            kMuestra := kMuestra + NPPorCiclo;
          while kMuestra > serie.n do
            kMuestra := kMuestra - NPPorCiclo;

          valor := Serie.e(kMuestra);

          //        writeln('kPaso: '+IntToStr(kPaso)+' jCiclo: '+IntToStr( jCiclo )+' jOverlap: '+IntToStr( jOverlap ) );
          vaux.pon_e(ipos, valor);
          Inc(ipos);
        end;

    // al ordenar creciente quedan los FILTRADOS al inicio.
    // la primer muestra no filtrada queda en la posición cnt_Filtrados+1
    vaux.Sort(True);


    cnt_Filtrados := 0;

    for k := 1 to vaux.n do
    begin
      if vaux.e(k) <= UMBRAL_TOCADOS then
        Inc(cnt_Filtrados);
    end;

    if (umbralFiltro > UMBRAL_TOCADOS) and Filtrar then
    begin
      for k := 1 to vaux.n do
      begin
        if vaux.e(k) <= umbralFiltro then
          Inc(cnt_Filtrados);
      end;
    end;

    nNoFiltrados := vaux.n - cnt_Filtrados;

    if nNoFiltrados = 0 then
      raise Exception.Create('Imposible continuar, demasiados huecos!!');


    va := TVectR.Create_Init(NPuntosResultado);
    for k := 1 to va.n do
    begin
      jrd := k / nPuntosResultado * nNoFiltrados;
      va.pv[k] := vaux.interpol(jrd + cnt_Filtrados);
    end;
    VectorDeMuestras[kPaso - 1] := va;

  end;


  // Bien, ahora con los vectores de muestras vamos a construir las fuentes
  // aleatorias que generan valores con igual función de densidad de probabilidad
  // que la del vector de muestras.
  setlength(res, NPPorCiclo);
  for kPaso := 1 to NPPorCiclo do
    res[kPaso - 1] := Tf_ddp_VectDeMuestras.Create_SinClonarMuestras(
      VectorDeMuestras[kPaso - 1], nil, 31);
  Result := res;
end;



(*
rch@20140618

EN DESARROLLO, SIN PROBAR. Tendría que sustituir al de arriba.

Esta versión intenta tener en cuenta los tiempos reales entre muestras,
periodo del mi-ciclo y del ciclo itentando liverar la obligaciónd e que los
periodos tengan que ser un múltiplo exacto del intervalo entre meustras.


*)
function uniformizadores_dt(
  serie: TVectR; NPPorCiclo: integer;
  overlapping: integer;
  traslapping: integer;
  filtrar: boolean;
  umbralFiltro: NReal;
  NPuntosResultado: integer;
  dt_Muestreo,
  dt_MiniCiclo,
  dt_Ciclo: double
  ): TDAOf_ddp_VectDeMuestras;

var
  kPaso, kMuestra: integer;
  nCiclos: integer;
  VectorDeMuestras: array of TVectR;
  res: TDAOf_ddp_VectDeMuestras;
  jOverlap, jTraslap: integer;
  vaux: TDAOfVectR;
  cntTocados: TDAOfNInt;
  va: TVectR;
  valor: NReal;
  cnt_Filtrados: integer;
  k: integer;

  jrd: NReal;
  kDeformador: integer;

  MaxNMuestras: integer;

begin
  // Calculamos la cantidad de ciclos enteros que caben en la serie
  nCiclos := Serie.n div NPPorCiclo;

  // Vamos a hacer un Vector de Muestras para cada paso dentro de un ciclo
  setlength(VectorDeMuestras, NPPorCiclo);

  // Vectores auxiliares para soporte del conjunto ampliado de muestras
  setlength(vaux, NPPorCiclo);

  // para llevar la cantidad de muestras agregada a cada lente
  setlength(cntTocados, NPPorCiclo);

  MaxNMuestras := (nCiclos + 1) * (1 + 2 * overlapping) * (1 + 2 * traslapping);

  // inicializamos los vectores y los contadores de muestras usadas.
  for kPaso := 0 to NPPorCiclo - 1 do
  begin
    // creamos vectores con el máximo de muestras que pueden tener los deformadores
    vaux[kPaso] := TVectR.Create_Init(MaxNMuestras);
    cntTocados[kPaso] := 0;
  end;


  cnt_filtrados := 0;


  // Ahora recorremos las muestras y según los tiempos, el overlapin y traslaping
  // colocamos cada muestra en los deformadores que correspondan

  for kMuestra := 1 to serie.n do
  begin

    valor := Serie.e(kMuestra);

    if filtrar and (valor <= umbralFiltro) then
    begin
      Inc(cnt_Filtrados);
    end
    else
    begin
      for jTraslap := -Traslapping to Traslapping do
        for jOverlap := -Overlapping to Overlapping do
        begin
          kDeformador := trunc(
            frac( ( (kMuestra - 1 + jOverlap) * dt_Muestreo + jTraslap * dt_MiniCiclo ) / dt_Ciclo) * (NPPorCiclo - 1) + 0.5);

          if kDeformador < 0 then
            kDeformador :=
              trunc(((frac((kMuestra - 1 + jOverlap) * dt_Muestreo + jTraslap * dt_MiniCiclo) / dt_Ciclo) +
              1) * (NPPorCiclo - 1) + 0.5);

          if kDeformador > high(vaux) then
            kDeformador :=
              trunc(((frac((kMuestra - 1 + jOverlap) * dt_Muestreo + jTraslap * dt_MiniCiclo) / dt_Ciclo) -
              1) * (NPPorCiclo - 1) + 0.5);

          if (kDeformador > 0) and (kDeformador > high(vaux)) and
            (cntTocados[kDeformador] < MaxNMuestras) then
          begin
            Inc(cntTocados[kDeformador]);
            vaux[kDeformador].pon_e(cntTocados[kDeformador], valor);
          end;
        end;
    end;
  end;


  for kDeformador := 0 to NPPorCiclo - 1 do
  begin
    vaux[kDeformador].resize(cntTocados[kDeformador]);
    if vaux[kDeformador].n = 0 then
      raise Exception.Create('Imposible continuar, demasiados huecos!!');
    vaux[kDeformador].Sort(True);

    va := TVectR.Create_Init(NPuntosResultado);
    for k := 1 to NPuntosResultado do
    begin
      jrd := k / nPuntosResultado * vaux[kDeformador].n;
      va.pv[k] := vaux[kDeformador].interpol(jrd);
    end;

    VectorDeMuestras[kDeformador] := va;
    vaux[kDeformador].Free;
  end;
  setlength(vaux, 0);
  setlength(cntTocados, 0);

  // Bien, ahora con los vectores de muestras vamos a construir las fuentes
  // aleatorias que generan valores con igual función de densidad de probabilidad
  // que la del vector de muestras.
  setlength(res, NPPorCiclo);
  for kDeformador := 0 to NPPorCiclo - 1 do
    res[kDeformador] := Tf_ddp_VectDeMuestras.Create_SinClonarMuestras(
      VectorDeMuestras[kDeformador], nil, 31);
  Result := res;
end;




function ivertirfiltro(filtro: TVectR; series: array of TVectR;
  kSerie: integer; filtrar: boolean; umbralFiltro: NReal;
  var varianzaDelResultado: NReal; var cntSecuenciasUsadas: integer): TVectR;
var
  k: integer;
  orden: integer;
  res: TVectR;
  a, av: NReal;
  h: integer;
  jSerie: integer;
  NSeries: integer;
  NPuntos: integer;
  flgFiltrado: boolean;

begin
  NSeries := length(series);
  orden := filtro.n div NSeries;
  NPuntos := series[0].n;
  res := TVectR.Create_Init(NPuntos - orden);

  varianzaDelResultado := 0;
  cntSecuenciasUsadas := 0;

  if not Filtrar then
  begin
    for k := 1 + orden to NPuntos do
    begin
      a := series[kSerie - 1].pv[k];
      for jSerie := 1 to NSeries do
        for h := 1 to orden do
          if (k - h) > 0 then
            a := a - filtro.e((h - 1) * NSeries + jSerie) * series[jSerie - 1].pv[k - h];
      res.pv[k - orden] := a;
      varianzaDelResultado := a * a + varianzaDelResultado;
      Inc(cntSecuenciasUsadas);
    end;
  end
  else
  begin
    for k := 1 + orden to NPuntos do
    begin
      a := series[kSerie - 1].pv[k];
      flgFiltrado := a <= umbralFiltro;
      for jSerie := 1 to NSeries do
        for h := 1 to orden do
          if ((k - h) > 0) and (not FlgFiltrado) then
          begin
            av := series[jSerie - 1].pv[k - h];
            if av > umbralFiltro then
              a := a - filtro.e((h - 1) * NSeries + jSerie) * av
            else
              flgFiltrado := True;
          end;

      if flgFiltrado then
        res.pv[k - orden] := umbralFiltro
      else
      begin
        res.pv[k - orden] := a;
        varianzaDelResultado := a * a + varianzaDelResultado;
        Inc(cntSecuenciasUsadas);
      end;
    end;
  end;

  varianzaDelResultado := varianzaDelResultado / (cntSecuenciasUsadas - 1);
  Result := res;
end;


procedure Write_SeriesGaussianas(seriesDeDatos: TSeriesDeDatos);
begin
  seriesDeDatos.WriteToArchi('series_gaussianas.xlt');
end;



procedure Write_FFTSeriesGaussianas(seriesDeDatos: TSeriesDeDatos);
var
  sal: textfile;
  kSerie, kPunto: integer;
  coA, coB: TDAOfNreal;
  A0: NReal;
begin
  setlength(coA, seriesDeDatos.NPuntos div 2);
  setlength(coB, seriesDeDatos.NPuntos div 2);
  WRFFTI01.Init(seriesDeDatos.NPuntos);
  for kSerie := 1 to seriesDeDatos.NSeries do
  begin
    WRFFTF01.FFTF(seriesDeDatos.Series[kSerie - 1].pv[1], A0, coA[0], coB[0]);
    assignfile(sal, 'FFT_seriegaussiana_' + IntToStr(kSerie) + '.txt');
    rewrite(sal);
    writeln(sal, 'A0: ', A0);
    writeln(sal, 'k', #9, 'CA', #9, 'CB');
    for kPunto := 0 to high(coA) do
      writeln(sal, kPunto + 1, #9, coA[kPunto], #9, coB[kPunto]);
    closefile(sal);
  end;
  WRFFTI01.done;
  setlength(coA, 0);
  setlength(coB, 0);
end;

// calcula los deformadores y el filtro óptimo en el espacio gaussiano.
// los lentes son por paso mientras que el filtro es único.
// La versión Analizar2 identifica un filtro para cada paso.
function Analizar(seriesDeDatos: TSeriesDeDatos; ordenfiltro: integer;
  overlapping: integer; traslapping: integer; nPuntosPorMiniciclo: integer;
  horasEntreMuestras: integer; FiltrarMenores: boolean; umbralFiltro: NReal;
  kMetodoCalculoB: integer; NDigitosDeformadores, NDecimalesDeformadores:
  integer // formato escritura
  ): NReal;

var
  kpunto: integer;
  kPaso: integer;
  //  r: string;
  ruidos: TDAOfVectR;
  kserie: integer;
  u, y: NReal;
  k, j, h: integer;
  //  sal: TextFile;
  sintetizadorSimSEE: TModeloSintetizadorCEGH;
  NPFD: integer;
  nVEred: integer;
  varNoExplicada: NReal;
  d_varNoExplicada: NReal;
  cntSecuenciasUsadas: integer;
  BtB: TMatR;
begin
  // Defimimos la cantidad de puntos que queremos sobre las funciones
  // desformantes.
  NPFD := 200;

  nVEred := seriesDeDatos.NSeries * ordenfiltro;

  // creamos el sintetizadorSimSEE
  sintetizadorSimSEE := TModeloSintetizadorCEGH.Create(
    seriesDeDatos.NSeries, seriesDeDatos.nombresSeries, ordenfiltro,
    seriesDeDatos.NPPorCiclo, NPFD, horasEntreMuestras, nVEred);

(* Para cada serie, calculamos los uniformizadores.
Cada casillero de FX tiene un vector de fuentes aleatorias
una para cada punto de un ciclo con igual histograma que el grupo
de puntos de la serie original para ese punto del ciclo *)
  for kserie := 1 to seriesDeDatos.NSeries do
    sintetizadorSimSEE.funcionesDeformantes[kserie - 1] :=
      uniformizadores(seriesDeDatos.series[kserie - 1],
      seriesDeDatos.NPPorCiclo, overlapping, traslapping, NPuntosPorMiniciclo,
      FiltrarMenores, umbralFiltro, NPFD);



  // ahora transformamos los datos de las series obteniendo así
  // las señales gaussianas con las que podremos hacer la identificación
  // lineal.
  for kPunto := 1 to seriesDeDatos.NPuntos do
  begin
    for kSerie := 1 to seriesDeDatos.NSeries do
    begin
        (* Con el número de serie y el paso dentro del ciclo, identificamos
        la fuente aleatoria cuya función de distribución de prob. podemos
        usar para transformar la serie de datos en una uniforme *)
      kPaso := ((kPunto - 1) mod seriesDeDatos.NPPorCiclo) + 1;
      u := seriesDeDatos.Series[kSerie - 1].e(kPunto);
      if (not FiltrarMenores) or (u > umbralFiltro) then
      begin
        y := SintetizadorSimSEE.xTog(u, kSerie, kPaso);
        (* Guardamos el valor transformado en la misma serie *)
        seriesDeDatos.Series[kSerie - 1].pon_e(kPunto, y);
      end;
    end;
  end;


  write_SeriesGaussianas(seriesDeDatos);
  //  Write_FFTSeriesGaussianas( seriesDeDatos );

  // verificamos que las varianzas de las series trasformadas
  // sean la unidad
  writeln(' Varianza y promedio de las series gaussianadas. Deberían ser 1 y 0 respectivamente.'
    );
  for kSerie := 1 to seriesDeDatos.NSeries do
  begin
    u := seriesDeDatos.Series[kSerie - 1].promedio_filtrando(umbralFiltro);
    y := seriesDeDatos.Series[kSerie - 1].varianza_filtrando(umbralFiltro);

    writeln(' varianza xg(', kSerie, '): ', y, ' , prom: ', u);

    // como puede pasar que el overlapping y el traslapping hagan que
    // la realización particular no tenga varianza 1 y eso afecta el cálculo
    // de las matrices A y B del filtro escalamos las series para llevarlas a varianza 1
    seriesDeDatos.Series[kSerie - 1].MasReal_filtrando(umbralFiltro, -u);
    seriesDeDatos.Series[kSerie - 1].PorReal_filtrando(umbralFiltro, 1 / sqrt(y));
  end;



  (**************************************************************)
  (* Identificación del filtro lineal (en el espacio gaussiano) *)
  (*------------------------------------------------------------*)
  // Primero identificamos la matriz A del filtro por mínimos cuadrados
  SintetizadorSimSEE.A_cte := calcmincuad_mrh(seriesDeDatos.Series,
    ordenfiltro, filtrarMenores, umbralFiltro, BtB).Crear_transpuesta;


  setlength(ruidos, seriesDeDatos.NSeries);
  // inversión de las series por el filtro
  // para tener las gaussianas de entrada
  // El largo de las series de ruidos es NPuntos-Orden pues
  // para los primeros puntos de las series (de 1 a Orden) no hay
  // datos para calcular el ruido.

    (*
    varNoExplicada := 0;
    for kserie := 1 to seriesDeDatos.NSeries do
    begin
      ruidos[kserie - 1] := ivertirfiltro(SintetizadorSimSee.A_cte.fila(kserie),
        seriesDeDatos.series, kserie, filtrarMenores, umbralFiltro,
        d_varNoExplicada, cntSecuenciasUsadas);
      varNoExplicada     := varNoExplicada + d_varNoExplicada / seriesDeDatos.NSeries;
    end;
    *)
  varNoExplicada := BtB.traza / seriesDeDatos.NSeries;

  if kMetodoCalculoB = 0 then
  begin
    BtB.WriteArchiXLT('btb.xlt');
    SintetizadorSimSee.B_cte := BtB.raiz_Cholesky;
  end
  else
  begin
    // OJO, como efecto secundario el dscuadrador puede acortar los vectores
    // de ruidos, eliminando los que fueron filtrados.
    if kMetodoCalculoB = 2 then
      SintetizadorSimSee.B_cte :=
        descuadrador_Cholesky(ruidos, FiltrarMenores, umbralFiltro)
    else
      SintetizadorSimSee.B_cte := descuadrador(ruidos, FiltrarMenores, umbralFiltro);
  end;
  BtB.Free;


  // bien ahora vamos a completar la información de reducción de estados.
  for j := 1 to ordenfiltro do
    for k := 1 to seriesDeDatos.NSeries do
    begin

      if ordenfiltro > 1 then
        SintetizadorSimSEE.nombreVarE[(k - 1) + (j - 1) * SeriesDeDatos.NSeries] :=
          seriesDeDatos.nombresSeries[k - 1] + '_' + IntToStr(j)
      else
        SintetizadorSimSEE.nombreVarE[(k - 1) + (j - 1) * SeriesDeDatos.NSeries] :=
          seriesDeDatos.nombresSeries[k - 1];

      SintetizadorSimSEE.MRed.pon_e(k, (k - 1) * ordenfiltro + j, 1);
      SintetizadorSimSEE.nDiscsVsE[(k - 1) * ordenfiltro + j - 1] := 5;
      setlength(SintetizadorSimSEE.ProbsVsE[(k - 1) * ordenfiltro + j - 1], 5);
      for h := 0 to 4 do
        SintetizadorSimSEE.ProbsVsE[(k - 1) * ordenfiltro + j - 1][h] := 0.2;
    end;

  SintetizadorSimSEE.WriteToArchi('SintetizadorCEGH.txt', NDigitosDeformadores,
    NDecimalesDeformadores);
  Result := VarNoExplicada;
end;


// construye los deformadores (uno por paso) y los filtros óptimos
// para cada paso en el espacio gaussiano.
function Analizar2(seriesDeDatos: TSeriesDeDatos; ordenfiltro: integer;
  overlapping: integer; traslapping: integer; NPuntosPorMiniciclo: integer;
  horasEntreMuestras: integer; FiltrarMenores: boolean; umbralFiltro: NReal;
  NDigitosDeformadores, NDecimalesDeformadores: integer // formato escritura
  ): NReal;

var
  kpunto: integer;
  kPaso: integer;
  //  r: string;
  //  ruidos: TDAOfVectR;
  kserie: integer;
  u, y: NReal;
  k, j, h: integer;
  //  sal: TextFile;

  NPFD: integer;
  nVEred: integer;

  sintetizadorSimSEE: TModeloSintetizadorCEGH;

  mcA, mcB: TDAOfMatR;
  cnt_fallos: integer;

  fsal: textfile;
  kfil, jcol: integer;
  varNoExplicado: NReal;

begin
  // Defimimos la cantidad de puntos que queremos sobre las funciones
  // desformantes.
  NPFD := 200;

  nVEred := seriesDeDatos.NSeries * ordenfiltro;

  // creamos el sintetizadorSimSEE
  sintetizadorSimSEE := TModeloSintetizadorCEGH.Create_MultiCiclo(
    seriesDeDatos.NSeries, seriesDeDatos.nombresSeries, ordenfiltro,
    seriesDeDatos.NPPorCiclo, NPFD, horasEntreMuestras, nVERed);

(* Para cada serie, calculamos los uniformizadores.
Cada casillero de FX tiene un vector de fuentes aleatorias
una para cada punto de un ciclo con igual histograma que el grupo
de puntos de la serie original para ese punto del cichlo *)
  for kserie := 1 to seriesDeDatos.NSeries do
    sintetizadorSimSEE.funcionesDeformantes[kserie - 1] :=
      uniformizadores(seriesDeDatos.series[kserie - 1],
      seriesDeDatos.NPPorCiclo, overlapping, traslapping, nPuntosPorMiniciclo,
      FiltrarMenores, umbralFiltro, NPFD);



  // ahora transformamos los datos de las series obteniendo así
  // las señales gaussianas con las que podremos hacer la identificación
  // lineal.
  for kPunto := 1 to seriesDeDatos.NPuntos do
  begin
    for kSerie := 1 to seriesDeDatos.NSeries do
    begin
        (* Con el número de serie y el paso dentro del ciclo, identificamos
        la fuente aleatoria cuya función de distribución de prob. podemos
        usar para transformar la serie de datos en una uniforme *)
      kPaso := ((kPunto - 1) mod seriesDeDatos.NPPorCiclo) + 1;
      u := seriesDeDatos.Series[kSerie - 1].e(kPunto);
      if (not FiltrarMenores) or (u > umbralFiltro) then
      begin
        y := SintetizadorSimSEE.xTog(u, kSerie, kPaso);
        (* Guardamos el valor transformado en la misma serie *)
        seriesDeDatos.Series[kSerie - 1].pon_e(kPunto, y);
      end;
    end;
  end;

  write_SeriesGaussianas(seriesDeDatos);



  // verificamos que las varianzas de las series trasformadas
  // sean la unidad
  writeln(' Varianza y promedio de las series gaussianadas. Deberían ser 1 y 0 respectivamente.'
    );
  for kSerie := 1 to seriesDeDatos.NSeries do
  begin
    u := seriesDeDatos.Series[kSerie - 1].promedio_filtrando(umbralFiltro);
    y := seriesDeDatos.Series[kSerie - 1].varianza_filtrando(umbralFiltro);

    writeln(' varianza xg(', kSerie, '): ', y, ' , prom: ', u);

    // como puede pasar que el overlapping y el traslapping hagan que
    // la realización particular no tenga varianza 1 y eso afecta el cálculo
    // de las matrices A y B del filtro escalamos las series para llevarlas a varianza 1
    seriesDeDatos.Series[kSerie - 1].MasReal_filtrando(umbralFiltro, -u);
    seriesDeDatos.Series[kSerie - 1].PorReal_filtrando(umbralFiltro, 1 / sqrt(y));
  end;



  (**************************************************************)
  (* Identificación del filtro lineal (en el espacio gaussiano) *)
  (*------------------------------------------------------------*)
  // Primero identificamos la matriz A del filtro por mínimos cuadrados
  cnt_fallos := calcmincuad_mrh_mc(seriesDeDatos.Series, ordenfiltro,
    filtrarMenores, umbralFiltro, seriesDeDatos.NPPorCiclo, overlapping,
    mcA{%H-}, mcB{%H-}, True, seriesDeDatos.nombresSeries, VarNoExplicado{%H-});


  SintetizadorSimSEE.mcA := mcA;
  SintetizadorSimSEE.mcB := mcB;

  assignfile(fsal, 'mab.xlt');
  rewrite(fsal);
  k := 0;
  for kfil := 1 to mcA[k].nf do
    for jcol := 1 to mcA[k].nc do
      Write(fsal, #9, seriesDeDatos.nombresSeries[kfil - 1] + '-' +
        seriesDeDatos.nombresSeries[jcol - 1]);
  Write(fsal, #9, '|');


  for kfil := 1 to mcB[k].nf do
    for jcol := 1 to mcB[k].nc do
      Write(fsal, #9, seriesDeDatos.nombresSeries[kfil - 1] + '-R_' +
        seriesDeDatos.nombresSeries[jcol - 1]);
  writeln(fsal);

  for k := 0 to high(mcA) do
  begin
    Write(fsal, k);
    if mcA[k] <> nil then
    begin
      for kfil := 1 to mcA[k].nf do
        for jcol := 1 to mcA[k].nc do
          Write(fsal, #9, mcA[k].e(kfil, jcol));
      Write(fsal, #9, '|');
      if (mcB[k] <> nil) then
      begin
        for kfil := 1 to mcB[k].nf do
          for jcol := 1 to mcB[k].nc do
            Write(fsal, #9, mcB[k].e(kfil, jcol));
      end
      else
        Write(fsal, #9, 'MCB[' + IntToStr(k) + ']=NIL');
    end;
    writeln(fsal);
  end;
  closefile(fsal);

  Assignfile(fsal, 'matricesA.xlt');
  rewrite(fsal);
  for k := 0 to high(mcA) do
  begin
    writeln(fsal, 'paso: ', #9, k);
    if mcA[k] <> nil then
      mcA[k].WriteXLT(fsal)
    else
      writeln(fsal, 'SISTEMA NO INVERTIBLE');
  end;
  closefile(fsal);


  Assignfile(fsal, 'matricesB.xlt');
  rewrite(fsal);
  for k := 0 to high(mcB) do
  begin
    writeln(fsal, 'paso: ', #9, k);
    if mcB[k] <> nil then
      mcB[k].WriteXLT(fsal)
    else
      writeln(fsal, 'Falló CHOLESKY');
  end;
  closefile(fsal);



  // bien ahora vamos a completar la información de reducción de estados.
  for j := 1 to ordenfiltro do
    for k := 1 to seriesDeDatos.NSeries do
    begin

      if ordenfiltro > 1 then
        SintetizadorSimSEE.nombreVarE[(k - 1) + (j - 1) * SeriesDeDatos.NSeries] :=
          seriesDeDatos.nombresSeries[k - 1] + '_' + IntToStr(j)
      else
        SintetizadorSimSEE.nombreVarE[(k - 1) + (j - 1) * SeriesDeDatos.NSeries] :=
          seriesDeDatos.nombresSeries[k - 1];

      SintetizadorSimSEE.MRed.pon_e(k, (k - 1) * ordenfiltro + j, 1);
      SintetizadorSimSEE.nDiscsVsE[(k - 1) * ordenfiltro + j - 1] := 5;
      setlength(SintetizadorSimSEE.ProbsVsE[(k - 1) * ordenfiltro + j - 1], 5);
      for h := 0 to 4 do
        SintetizadorSimSEE.ProbsVsE[(k - 1) * ordenfiltro + j - 1][h] := 0.2;
    end;


  SintetizadorSimSEE.WriteToArchi('SintetizadorCEGH_mc.txt',
    NDigitosDeformadores, NDecimalesDeformadores);
  Result := varNoExplicado; // por ahora
end;

// igual a la funcion analizar comun pero permite elegir el nombre del archivo
// salida
function Analizar3(seriesDeDatos: TSeriesDeDatos; ordenfiltro: integer;
  overlapping: integer; traslapping: integer; nPuntosPorMiniciclo: integer;
  horasEntreMuestras: integer; FiltrarMenores: boolean; umbralFiltro: NReal;
  kMetodoCalculoB: integer; NDigitosDeformadores, NDecimalesDeformadores: integer;
  nombreArch: string // formato escritura
  ): NReal;

var
  kpunto: integer;
  kPaso: integer;
  //  r: string;
  ruidos: TDAOfVectR;
  kserie: integer;
  u, y: NReal;
  k, j, h: integer;
  //  sal: TextFile;
  sintetizadorSimSEE: TModeloSintetizadorCEGH;
  NPFD: integer;
  nVEred: integer;
  varNoExplicada: NReal;
  d_varNoExplicada: NReal;
  cntSecuenciasUsadas: integer;
  BtB: TMatR;
begin
  // Defimimos la cantidad de puntos que queremos sobre las funciones
  // desformantes.
  NPFD := 200;

  nVEred := seriesDeDatos.NSeries * ordenfiltro;

  // creamos el sintetizadorSimSEE
  sintetizadorSimSEE := TModeloSintetizadorCEGH.Create(
    seriesDeDatos.NSeries, seriesDeDatos.nombresSeries, ordenfiltro,
    seriesDeDatos.NPPorCiclo, NPFD, horasEntreMuestras, nVEred);

(* Para cada serie, calculamos los uniformizadores.
Cada casillero de FX tiene un vector de fuentes aleatorias
una para cada punto de un ciclo con igual histograma que el grupo
de puntos de la serie original para ese punto del ciclo *)
  for kserie := 1 to seriesDeDatos.NSeries do
    sintetizadorSimSEE.funcionesDeformantes[kserie - 1] :=
      uniformizadores(seriesDeDatos.series[kserie - 1],
      seriesDeDatos.NPPorCiclo, overlapping, traslapping, NPuntosPorMiniciclo,
      FiltrarMenores, umbralFiltro, NPFD);



  // ahora transformamos los datos de las series obteniendo así
  // las señales gaussianas con las que podremos hacer la identificación
  // lineal.
  for kPunto := 1 to seriesDeDatos.NPuntos do
  begin
    for kSerie := 1 to seriesDeDatos.NSeries do
    begin
        (* Con el número de serie y el paso dentro del ciclo, identificamos
        la fuente aleatoria cuya función de distribución de prob. podemos
        usar para transformar la serie de datos en una uniforme *)
      kPaso := ((kPunto - 1) mod seriesDeDatos.NPPorCiclo) + 1;
      u := seriesDeDatos.Series[kSerie - 1].e(kPunto);
      if (not FiltrarMenores) or (u > umbralFiltro) then
      begin
        y := SintetizadorSimSEE.xTog(u, kSerie, kPaso);
        (* Guardamos el valor transformado en la misma serie *)
        seriesDeDatos.Series[kSerie - 1].pon_e(kPunto, y);
      end;
    end;
  end;


  write_SeriesGaussianas(seriesDeDatos);
  //  Write_FFTSeriesGaussianas( seriesDeDatos );


  // verificamos que las varianzas de las series trasformadas
  // sean la unidad
  writeln(' Varianza y promedio de las series gaussianadas. Deberían ser 1 y 0 respectivamente.'
    );
  for kSerie := 1 to seriesDeDatos.NSeries do
  begin
    u := seriesDeDatos.Series[kSerie - 1].promedio_filtrando(umbralFiltro);
    y := seriesDeDatos.Series[kSerie - 1].varianza_filtrando(umbralFiltro);

    writeln(' varianza xg(', kSerie, '): ', y, ' , prom: ', u);

    // como puede pasar que el overlapping y el traslapping hagan que
    // la realización particular no tenga varianza 1 y eso afecta el cálculo
    // de las matrices A y B del filtro escalamos las series para llevarlas a varianza 1
    seriesDeDatos.Series[kSerie - 1].MasReal_filtrando(umbralFiltro, -u);
    seriesDeDatos.Series[kSerie - 1].PorReal_filtrando(umbralFiltro, 1 / sqrt(y));
  end;



  (**************************************************************)
  (* Identificación del filtro lineal (en el espacio gaussiano) *)
  (*------------------------------------------------------------*)
  // Primero identificamos la matriz A del filtro por mínimos cuadrados
  SintetizadorSimSEE.A_cte := calcmincuad_mrh(seriesDeDatos.Series,
    ordenfiltro, filtrarMenores, umbralFiltro, BtB).Crear_transpuesta;


  setlength(ruidos, seriesDeDatos.NSeries);
  // inversión de las series por el filtro
  // para tener las gaussianas de entrada
  // El largo de las series de ruidos es NPuntos-Orden pues
  // para los primeros puntos de las series (de 1 a Orden) no hay
  // datos para calcular el ruido.

  varNoExplicada := 0;
  for kserie := 1 to seriesDeDatos.NSeries do
  begin
    ruidos[kserie - 1] := ivertirfiltro(SintetizadorSimSee.A_cte.fila(kserie),
      seriesDeDatos.series, kserie, filtrarMenores, umbralFiltro,
      d_varNoExplicada, cntSecuenciasUsadas);
    varNoExplicada := varNoExplicada + d_varNoExplicada / seriesDeDatos.NSeries;
  end;


  if kMetodoCalculoB = 0 then
  begin
    BtB.WriteArchiXLT('btb.xlt');
    SintetizadorSimSee.B_cte := BtB.raiz_Cholesky;
  end
  else
  begin
    // OJO, como efecto secundario el dscuadrador puede acortar los vectores
    // de ruidos, eliminando los que fueron filtrados.
    if kMetodoCalculoB = 2 then
      SintetizadorSimSee.B_cte :=
        descuadrador_Cholesky(ruidos, FiltrarMenores, umbralFiltro)
    else
      SintetizadorSimSee.B_cte := descuadrador(ruidos, FiltrarMenores, umbralFiltro);
  end;
  BtB.Free;


  // bien ahora vamos a completar la información de reducción de estados.
  for j := 1 to ordenfiltro do
    k := seriesDeDatos.NSeries;
  SintetizadorSimSEE.nVE := 1;
  SetLength(SintetizadorSimSEE.nDiscsVsE, 1);
  SetLength(SintetizadorSimSEE.nombreVarE, 1);
  SintetizadorSimSEE.MRed := TMatR.Create_Init(1, SintetizadorSimSEE.nBornesSalida);


  if ordenfiltro > 1 then
    SintetizadorSimSEE.nombreVarE[(k - 1) + (j - 1) * SeriesDeDatos.NSeries] :=
      seriesDeDatos.nombresSeries[k - 1] + '_' + IntToStr(j)
  else
    SintetizadorSimSEE.nombreVarE[0] :=
      seriesDeDatos.nombresSeries[k - 1];
  SintetizadorSimSEE.MRed.pm[1] := SintetizadorSimSEE.A_cte.Crear_Columna(k);
  SintetizadorSimSEE.MRed.pm[1].HacerUnitario;

  SintetizadorSimSEE.nDiscsVsE[0] := 5;
  setlength(SintetizadorSimSEE.ProbsVsE[0], 5);

  SintetizadorSimSEE.ProbsVsE[0][0] := 0.06;
  SintetizadorSimSEE.ProbsVsE[0][1] := 0.09;
  SintetizadorSimSEE.ProbsVsE[0][2] := 0.20;
  SintetizadorSimSEE.ProbsVsE[0][3] := 0.30;
  SintetizadorSimSEE.ProbsVsE[0][4] := 0.35;

  SintetizadorSimSEE.WriteToArchi(nombreArch, NDigitosDeformadores,
    NDecimalesDeformadores);
  Result := VarNoExplicada;
end;

procedure TForm1.BBuscarClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
    eArchiSeriesDeDatos.Text := OpenDialog1.FileName;
end;


function TForm1.buscarAnio(anio: integer): integer;
var
  buscando: boolean;
  kres: integer;
begin
  kres := -1;
  buscando := True;
  while buscando and (kBuscarDesde < length(aniosanalogos)) do
    if aniosanalogos[kBuscarDesde] = anio then
    begin
      buscando := False;
      kres := kBuscarDesde;
    end
    else
    if aniosanalogos[kBuscarDesde] < anio then
      Inc(kBuscarDesde)
    else
      buscando := False;

  Result := kres;

end;

function TForm1.Mascarado(anio, semana: integer): boolean;
var
  anioInicio: integer;
begin
  if semana >= self.semanaDeInicio then
    anioInicio := anio
  else
    anioInicio := anio - 1;
  Result := buscarAnio(anioInicio) < 0;
end;


procedure TForm1.MascararAniosNoAnalogos(datos: TSeriesDeDatos; mascara: double);
var
  kserie: integer;
  anio, isem, ksem: integer;
begin
  for kserie := 0 to high(datos.series) do
  begin
    anio := datos.PM_Anio;
    kBuscarDesde := 0;
    for isem := 1 to datos.series[0].n do
    begin
      // supongo que empiezan las series en la primer semana del año.
      ksem := ((isem - 1) mod 52) + 1;
      if mascarado(anio, ksem) then
        datos.series[kserie].pon_e(isem, mascara);
      if ksem = 52 then
        Inc(anio);
    end;
  end;
end;


function sino(x: boolean): string;
begin
  if x then
    Result := 'Si'
  else
    Result := 'No';
end;

procedure TForm1.BAnalizarClick(Sender: TObject);
var
  varNoExplicada: NReal;

begin
  if FileExists(eArchiSeriesDeDatos.Text) then
  begin
    if seriesDeDatos <> nil then
      seriesDeDatos.Free;

    seriesDeDatos := TSeriesDeDatos.CreateFromArchi(eArchiSeriesDeDatos.Text);

    if self.cbAniosAnalogos.Checked then
    begin
      GetAniosAnalogos;
      MascararAniosNoAnalogos(seriesDeDatos, strToFloat(eUmbralFiltro.Text));
    end;

    if not cbFiltroCiclico.Checked then
    begin
      varNoExplicada := Analizar(seriesDeDatos, StrToInt(
        eOrdenDelFiltro.Text), StrToInt(eOverlapping.Text),
        StrToInt(eTraslapping.Text), StrToInt(eNPuntosPorMiniciclo.Text),
        StrToInt(eHorasEntreMuestras.Text), cbFiltrarMenores.Checked,
        strToFloat(eUmbralFiltro.Text), self.rbg_calculoB.ItemIndex,
        StrToInt(eNDigitosDesformadores.Text), StrToInt(eNDecimalesDesformadores.Text));
    end
    else
    begin
      varNoExplicada := Analizar2(seriesDeDatos, StrToInt(
        eOrdenDelFiltro.Text), StrToInt(eOverlapping.Text),
        StrToInt(eTraslapping.Text), StrToInt(eNPuntosPorMiniciclo.Text),
        StrToInt(eHorasEntreMuestras.Text), cbFiltrarMenores.Checked,
        strToFloat(eUmbralFiltro.Text), StrToInt(eNDigitosDesformadores.Text),
        StrToInt(eNDecimalesDesformadores.Text));
    end;
    eVarianzaDeLoNoExplicado.Text := FloatToStr(varNoExplicada);

    if seriesDeDatos <> nil then
    begin
      seriesDeDatos.Free;
      seriesDeDatos := nil;
    end;
  end
  else
    ShowMessage('Nombre de Archivo Inválido: ' + eArchiSeriesDeDatos.Text);
end;

procedure TForm1.ESHYClick(Sender: TObject);
const
  ceB = 0.62739306;
  ceP = 0.24865988;
  ceS = 0.12394705;
var
  varNoExplicada: NReal;
  archi: string;
  vectempH: TVectR;
  i, Totalseries, j, k, kpunto, largo: integer;
begin
  if FileExists(eArchiSeriesDeDatos.Text) then
  begin
    if seriesDeDatos <> nil then
      seriesDeDatos.Free;

    for i := 1 to StrToInt(eOrdenESHY.Text) do
    begin

      seriesDeDatos := TSeriesDeDatos.CreateFromArchi(eArchiSeriesDeDatos.Text);
      Totalseries := seriesDeDatos.NSeries;
      seriesDeDatos.NSeries := Totalseries + 1;
      setlength(seriesDeDatos.series, Totalseries + 1);
      setlength(seriesDeDatos.tipo_serie, Totalseries + 1);
      seriesDeDatos.series[Totalseries] := TVectR.Create_Init(seriesDeDatos.NPuntos);
      seriesDeDatos.tipo_serie[Totalseries] := ENTRADA;
      vectempH := TVectR.Create_Init(seriesDeDatos.NPuntos);
      for kpunto := 1 to seriesDeDatos.NPuntos do
      begin
        vectempH.pv[kpunto] :=
          ceB * seriesDeDatos.series[0].pv[kpunto] + ceP *
          seriesDeDatos.series[1].pv[kpunto] + ceS *
          seriesDeDatos.series[2].pv[kpunto] * 2;
      end;

      seriesDeDatos.nombresSeries.Add('H_' + IntToStr(i));

      for kpunto := 1 to seriesDeDatos.NPuntos do
      begin
        for j := 0 to i - 1 do
        begin
          if (kpunto - j) >= 0 then
          begin
            seriesDeDatos.series[Totalseries].pv[kpunto] :=
              seriesDeDatos.series[Totalseries].pv[kpunto] +
              vectempH.pv[kpunto - j];
          end;
        end;
        if kpunto < i then
          k := kpunto
        else
          k := i;
        seriesDeDatos.series[Totalseries].pv[kpunto] :=
          seriesDeDatos.series[Totalseries].pv[kpunto] / (k);
      end;
      seriesDeDatos.WriteToArchi(eArchiSeriesDeDatos.Text +
        'serie_LikeESHY_H_' + IntToStr(i) + '.txt');
      //seriesDeDatos.n
      if self.cbAniosAnalogos.Checked then
      begin
        GetAniosAnalogos;
        MascararAniosNoAnalogos(seriesDeDatos, strToFloat(eUmbralFiltro.Text));
      end;
      archi := eArchiSeriesDeDatos.Text;
      largo := Length(archi) - 4;
      //archi:=Delete(archi,largo,4);
      archi := 'CEGH_' + archi + '_LikeESHY_H_' + IntToStr(i) + '.txt';
      if not cbFiltroCiclico.Checked then
      begin
        varNoExplicada :=
          Analizar3(seriesDeDatos, StrToInt(eOrdenDelFiltro.Text),
          StrToInt(eOverlapping.Text), StrToInt(eTraslapping.Text),
          StrToInt(eNPuntosPorMiniciclo.Text), StrToInt(eHorasEntreMuestras.Text),
          cbFiltrarMenores.Checked, strToFloat(eUmbralFiltro.Text),
          self.rbg_calculoB.ItemIndex, StrToInt(eNDigitosDesformadores.Text),
          StrToInt(eNDecimalesDesformadores.Text), archi);
      end
      else
      begin
        varNoExplicada :=
          Analizar2(seriesDeDatos, StrToInt(eOrdenDelFiltro.Text),
          StrToInt(eOverlapping.Text), StrToInt(eTraslapping.Text),
          StrToInt(eNPuntosPorMiniciclo.Text),
          StrToInt(eHorasEntreMuestras.Text), cbFiltrarMenores.Checked,
          strToFloat(eUmbralFiltro.Text), StrToInt(
          eNDigitosDesformadores.Text), StrToInt(
          eNDecimalesDesformadores.Text));
      end;

    end;

    //eVarianzaDeLoNoExplicado.Text := FloatToStr(varNoExplicada);

    if seriesDeDatos <> nil then
    begin
      seriesDeDatos.Free;
      seriesDeDatos := nil;
    end;
  end
  else
    ShowMessage('Nombre de Archivo Inválido: ' + eArchiSeriesDeDatos.Text);

end;

procedure TForm1.BLeerDatosBasicosClick(Sender: TObject);
begin
  if seriesDeDatos <> nil then
    seriesDeDatos.Free;
  seriesDeDatos := TSeriesDeDatos.CreateFromArchi(eArchiSeriesDeDatos.Text);
end;



procedure TForm1.BPredictorNino34Click(Sender: TObject);
var
  predictor: TModeloSintetizadorCEGH;
  //Es de dimensión NPasos X NCronicas.
  //Guarda para cada paso hacia adelante los resultados de los sorteos de cada crónica
  pronostico: array of TVectR;
  NCronicas: integer;
  NPasos: integer;
  kPaso: integer;
  iCronica: integer;
  serieHistorica: TSeriesDeDatos;
  y, x, r: TVectR;
  //Son de dimensión NPasos X (NDatos + NPasos)
  //Guardan para cada paso hacia adelante los valores medios, valores con
  //probabilidad 20% y valores con probabilidad 80% de los resultados obtenidos
  //en las crónicas de pronóstico
  vm, p20, p80: TMatR;
  f: textFile;
  iDesp: integer;
  NDatos: integer;
  NOrdenFiltro: integer;
  vy: NREal;
  i, j: integer;
begin
  serieHistorica := TSeriesDeDatos.CreateFromArchi(
    '\simsee\src-delphi\ax\analisisserial\serieNino34.txt');

  predictor := TModeloSintetizadorCEGH.CreateFromArchi(
    '\simsee\src-delphi\ax\analisisserial\SintCEGH-mensual-nino34-fo1.txt');

  x := predictor.CrearVector_EstadoX;
  y := predictor.CrearVector_Salida;
  r := predictor.CrearVector_RBG;

  NOrdenFiltro := predictor.calcOrdenDelFiltro;

  NDatos := serieHistorica.NPuntos;

  NCronicas := 1000;

  NPasos := 5;

  vm := TMatR.Create_Init(NPasos, NDatos + NPasos);
  p20 := TMatR.Create_Init(NPasos, NDatos + NPasos);
  p80 := TMatR.Create_Init(NPasos, NDatos + NPasos);

  setlength(pronostico, NPasos);
  for kPaso := 0 to high(pronostico) do
    pronostico[kPaso] := TVectR.Create_Init(NCronicas);

  for idesp := 1 to NDatos do
  begin
    for iCronica := 1 to NCronicas do
    begin
      // cargamos el vector de estados desde la serie de datos
      predictor.InicializarDesdeDatosReales(x, serieHistorica, idesp, 0);
      // ?? gaussianar

      for kPaso := 0 to NPasos - 1 do
      begin
        // sortear salidas de ruido blanco gaussiano
        predictor.SortearValores(r);

        // calcula y= Ax + Br
        predictor.CalcularProximasSalidas(y, x, r, 0);


      {$IFDEF DBG_ESTIMG}
        vy := y.pv[1];
      {$ELSE}
        // desgaussianar
        vy := predictor.gTox(y.pv[1], 1, ((idesp - 1 + kPaso) mod
          serieHistorica.NPPorCiclo) + 1);
      {$ENDIF}


        // cambia el estado
        predictor.EvolucionarEstado_(x, y);

        pronostico[kPaso].pon_e(iCronica, vy);
      end;
    end;

    for kPaso := 0 to NPasos - 1 do
    begin
      pronostico[kPaso].Sort(True);
      vm.pon_e(1 + kpaso, idesp + kpaso + 1, pronostico[kPaso].promedio);
      p20.pon_e(1 + kpaso, idesp + kpaso + 1, pronostico[kPaso].e(
        trunc(0.2 * NCronicas + 0.5)));
      p80.pon_e(1 + kpaso, idesp + kpaso + 1, pronostico[kPaso].e(
        trunc(0.8 * NCronicas + 0.5)));
    end;
  end;

  assignFile(f, 'PronosticoNino34-0712.xlt');
  rewrite(f);

  Write(f, #9, 'vr');
  for i := 1 to vm.nf do
    Write(f, #9, 'vm' + IntToStr(i));
  for i := 1 to vm.nf do
    Write(f, #9, 'p20' + IntToStr(i));
  for i := 1 to vm.nf do
    Write(f, #9, 'p80' + IntToStr(i));
  writeln(f);

  for j := 1 to vm.nc do
  begin
    Write(f, j);
    if j < NDatos then
    begin
      {$IFDEF DBG_ESTIMG}
      vy := predictor.xTog(serieHistorica.series[0].e(j), 1,
        ((j - 1) mod serieHistorica.NPPorCiclo) + 1);
      Write(f, #9, vy: 12: 4);
      {$ELSE}
      Write(f, #9, serieHistorica.series[0].e(j): 12: 4);
      {$ENDIF}
    end
    else
      Write(f, #9, '');

    for i := 1 to vm.nf do
      Write(f, #9, vm.e(i, j): 12: 4);

    for i := 1 to vm.nf do
      Write(f, #9, p20.e(i, j): 12: 4);

    for i := 1 to vm.nf do
      Write(f, #9, p80.e(i, j): 12: 4);

    writeln(f);
  end;
  closeFile(f);
end;

procedure TForm1.BTestVarianzaClick(Sender: TObject);
var
  predictor: TModeloSintetizadorCEGH;
  NCronicas: integer;
  x, y, r: TVectR;
  icronica: integer;
  y2: TVectR;
  j: integer;
begin
  if not OpenDialog1.Execute then
    exit;

  predictor := TModeloSintetizadorCEGH.CreateFromArchi(OpenDialog1.FileName);

(*
  predictor:= TModeloSintetizadorCEGH.CreateFromArchi(
    'C:\simsee\datos_comunes\sintetizadores\SintCEGH-mensual-nino34-fo3.txt');
    *)
  x := predictor.CrearVector_EstadoX;
  y := predictor.CrearVector_Salida;
  y2 := TVectR.Create_Clone(y);
  r := predictor.CrearVector_RBG;
  NCronicas := 1000000;


  // le damos un poco de manija al filtro para que quede cargado
  for iCronica := 1 to 1000 do
  begin
    predictor.SortearValores(r);
    predictor.CalcularProximasSalidas(y, x, r, 0);
    predictor.EvolucionarEstado_(x, y);
  end;


  // ahora empezamos a calcular las varianzas de las salidas.
  for iCronica := 1 to NCronicas do
  begin
    predictor.SortearValores(r);
    predictor.CalcularProximasSalidas(y, x, r, 0);
    for j := 1 to y.n do
      y2.pv[j] := y2.pv[j] + sqr(y.pv[j]);
    predictor.EvolucionarEstado_(x, y);
  end;

  for j := 1 to y.n do
    y2.pv[j] := y2.pv[j] / NCronicas;

  writeln('Varianza de las series sintéticas.');
  for j := 1 to y.n do
    writeln('ys[', j, ']: ', y2.pv[j]);
end;



procedure TForm1.Button1Click(Sender: TObject);
begin
  verdoc(self, 'analisisserial-manualdeusuario', '');
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  hurst: array of THurst_rescaled;
  kserie: integer;
  sal: textfile;
  j: integer;

begin
  if FileExists(eArchiSeriesDeDatos.Text) then
  begin
    if seriesDeDatos <> nil then
      seriesDeDatos.Free;
    seriesDeDatos := TSeriesDeDatos.CreateFromArchi(eArchiSeriesDeDatos.Text);

    setlength(hurst, seriesDeDatos.NSeries);
    assignfile(sal, 'hurst.xlt');
    rewrite(sal);

    writeln(sal, 'serie', #9, 'slope', #9, 'intercept');
    for kserie := 0 to seriesDeDatos.NSeries - 1 do
    begin
      hurst[kserie] := THurst_Rescaled.Create(
        @(seriesDeDatos.series[kserie].pv[1]), seriesDeDatos.series[kserie].n);
      writeln(sal, kserie + 1, #9, hurst[kserie].slope, #9, hurst[kserie].intercept);
    end;

    writeln(sal);

    Write(sal, 'logN');
    for kserie := 0 to seriesDeDatos.NSeries - 1 do
      Write(sal, #9, 'logRS_' + seriesDeDatos.nombresSeries[kserie]);
    writeln(sal);

    for j := 1 to hurst[0].points_x.n do
    begin
      Write(sal, hurst[0].points_x.pv[j]);
      for kserie := 0 to seriesDeDatos.NSeries - 1 do
        Write(sal, #9, hurst[kserie].points_y.pv[j]);
      writeln(sal);
    end;

    for kserie := 0 to seriesDeDatos.NSeries - 1 do
    begin
      hurst[kserie].Free;
    end;

    closefile(sal);
  end;
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  kserie: integer;
  sal: textfile;
  j: integer;

begin
  if FileExists(eArchiSeriesDeDatos.Text) then
  begin
    if seriesDeDatos <> nil then
      seriesDeDatos.Free;
    seriesDeDatos := TSeriesDeDatos.CreateFromArchi(eArchiSeriesDeDatos.Text);

    for kserie := 0 to seriesDeDatos.NSeries - 1 do
    begin
      //      seriesDeDatos.series[kserie].aplicar_hanning;
      AutoCorrelacion(seriesDeDatos.series[kserie].n,
        seriesDeDatos.series[kserie].pv[1]);
    end;
    assignfile(sal, 'autoco.xlt');
    rewrite(sal);

    Write(sal, 'k');
    for kserie := 0 to seriesDeDatos.NSeries - 1 do
      Write(sal, #9, 'r_' + seriesDeDatos.nombresSeries[kserie]);
    writeln(sal);

    for j := 1 to seriesDeDatos.series[0].n do
    begin
      Write(sal, j - 1);
      for kserie := 0 to seriesDeDatos.NSeries - 1 do
        Write(sal, #9, seriesDeDatos.series[kserie].pv[j]);
      writeln(sal);
    end;

    wrffti01.done;
    closefile(sal);
  end;
end;

procedure TForm1.Button4Click(Sender: TObject);
var
  kserie: integer;
  sal: textfile;
  j: integer;
  ncoefs: integer;
  vm: NReal;
begin
  if FileExists(eArchiSeriesDeDatos.Text) then
  begin
    if seriesDeDatos <> nil then
      seriesDeDatos.Free;
    seriesDeDatos := TSeriesDeDatos.CreateFromArchi(eArchiSeriesDeDatos.Text);

    for kserie := 0 to seriesDeDatos.NSeries - 1 do
    begin
      vm := seriesDeDatos.series[kserie].promedio;
      seriesDeDatos.series[kserie].MasReal(-vm);
      seriesDeDatos.series[kserie].aplicar_hanning;
      h2f(seriesDeDatos.series[kserie].n, seriesDeDatos.series[kserie].pv[1]);
    end;

    assignfile(sal, 'h2f.xlt');
    rewrite(sal);

    Write(sal, 'k');
    for kserie := 0 to seriesDeDatos.NSeries - 1 do
      Write(sal, #9, 'h2_' + seriesDeDatos.nombresSeries[kserie]);
    writeln(sal);

    nCoefs := (seriesDeDatos.series[0].n div 2) + 1;
    for j := 1 to nCoefs do
    begin
      Write(sal, j - 1);
      for kserie := 0 to seriesDeDatos.NSeries - 1 do
        Write(sal, #9, seriesDeDatos.series[kserie].pv[j]);
      writeln(sal);
    end;

    wrffti01.done;
    closefile(sal);
  end;
end;


procedure TForm1.GetAniosAnalogos;
var
  s: string;
  k: integer;
  anio: integer;
begin
  s := trim(self.lstAniosAnalogos.Text);
  if s = '' then
  begin
    setlength(aniosanalogos, 0);
    exit;
  end;
  setlength(aniosanalogos, 10000);

  k := 0;
  while s <> '' do
  begin
    anio := nextint(s);
    aniosAnalogos[k] := anio;
    Inc(k);
  end;
  setlength(aniosAnalogos, k);

  semanaDeInicio := StrToInt(eSemanaInicioVentanaAnaloga.Text);
  kBuscarDesde := 0;
end;

procedure TForm1.BCalcularPromediosMensualesClick(Sender: TObject);
var
  seriesDeDatos: TSeriesDeDatos;
  vmm: array of TVectR;
  cnts: array of TVectE;
  dt, dtbase: TDateTime;

  NDatos: integer;
  ipaso: integer;
  dtpaso: TDateTime;
  anio, mes, dia: word;
  v: NReal;
  filtrar: boolean;
  umbralF: NReal;
  flgPasa: boolean;
  fsal: textfile;
  iserie: integer;

begin
  if cbFiltrarMenores.Checked then
  begin
    filtrar := True;
    umbralF := StrToFloat(eUmbralFiltro.Text);
  end
  else
    filtrar := False;

  seriesDeDatos := TSeriesDeDatos.CreateFromArchi(eArchiSeriesDeDatos.Text);

  setlength(vmm, seriesDeDatos.NSeries);
  setlength(cnts, seriesDeDatos.NSeries);
  for iserie := 0 to seriesDeDatos.NSeries - 1 do
  begin
    vmm[iserie] := TVectR.Create_Init(12);
    cnts[iserie] := TVectE.Create_Init(12);
  end;

  NDatos := seriesDeDatos.series[0].n;
  dt := EncodeDate(seriesDeDatos.PM_Anio, seriesDeDatos.PM_Mes, seriesDeDatos.PM_Dia);
  dtbase := dt + (seriesDeDatos.PM_Hora + (seriesDeDatos.PM_Minuto +
    seriesDeDatos.PM_segundo / 60) / 60) / 24;
  dtpaso := seriesDeDatos.PeriodoDeMuestreo_horas / 24.0;

  for iPaso := 1 to NDatos do
  begin
    for iSerie := 0 to seriesDeDatos.NSeries - 1 do
    begin
      flgPasa := True;
      v := seriesDeDatos.series[iSerie].e(ipaso);
      if filtrar then
        flgPasa := v > umbralF;

      if flgPasa then
      begin
        dt := dtbase + (ipaso - 1) * dtpaso;
        DecodeDate(dt, anio, mes, dia);
        vmm[iSerie].acum_e(mes, v);
        cnts[iSerie].acum_e(mes, 1);
      end;
    end;
  end;

  assignfile(fsal, 'promedios_mensuales.xlt');
  rewrite(fsal);
  writeln(fsal, DateTimeToStr(now()));
  writeln(fsal, 'ANALISIS_SERIAL_PROMEDIOS_MENSUALES');
  writeln(fsal, 'Serie de datos: ' + eArchiSeriesDeDatos.Text);
  writeln(fsal, 'Valores esperados por mes:::::');
  for mes := 1 to 12 do
  begin
    Write(fsal, mes: 8);
    for iserie := 0 to SeriesDeDatos.NSeries - 1 do
    begin
      if cnts[iserie].pv[mes] > 0 then
      begin
        vmm[iserie].pv[mes] := vmm[iserie].pv[mes] / cnts[iserie].pv[mes];
        Write(#9, vmm[iserie].pv[mes]: 12: 3);
      end
      else
        Write(fsal, #9, '-SIN VALOR-');
    end;
    writeln(fsal);
  end;
  closefile(fsal);
end;



procedure TForm1.TestDistribucionGaussianaClick(Sender: TObject);
begin
  Test_DistribucionNormalCanonica;
end;

procedure TForm1.GenerarRuidoRectificadoClick(Sender: TObject);
(*

FuenteRuido( 0, 1, AmpLitudRuido );
R( 1, 2 , 1E3 ohm );
C( 0, 2, 0.5E-6 F );
R( 0, 2, 1E3 ohm );
dt= 1e-3 segs;
salida= tensión(0,2)
*)

const
  NPuntos = 1024 * 4;
var
  k: integer;
  uk: NReal;
  Vk, Vks: NReal;
  dt, r1, r2, C: NReal;
  a1, a2: NReal;
  sal: textfile;
begin
  dt := 1E-3;
  r1 := 1E3;
  r2 := 1E3;
  C := 20E-6;
  a1 := 1 - dt / r2 / C;
  a2 := dt / r1 / C;
  Vk := 0;

  assignfile(sal, 'series_ruidorectificado.xls');
  rewrite(sal);
  writeln(sal, 1, #9, 'NSeries');
  writeln(sal, NPuntos, #9, 'NPuntos');
  writeln(sal, 1, #9, 'Puntos por ciclo');
  writeln(sal, #9, 'Ruido');
  for k := 0 to NPuntos - 1 do
  begin
    Vks := a1 * Vk;
    uk := (random - 0.5) * 1000;
    if uk > Vk then
      Vks := Vks + a2 * (uk - Vk);
    writeln(sal, k, #9, Vks: 12: 1);
    Vk := Vks;
  end;
  closefile(sal);
end;

initialization
{$IFDEF FPC-LCL}
  {$I uanalisisserial.lrs}
{$ENDIF}
end.
