unit uTSumComb;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  uCosa, SysUtils, StdCtrls, uGlobs, uActores, uFechas, unodos, uCosaConNombre,
  uFichasLPD, usimplex, xMatDefs, uauxiliares, unodocombustible,
  uFuentesAleatorias;


resourcestring
  rsSuministroSimpleCombustible = 'Suministro Simple Combustible';


type
  TSuministroCombustible = class(TActor)
    NodoCombustible: TNodoCombustible;
    function ReferenciaAlNodo(NodoSumComb: TNodoCombustible): boolean; virtual;
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
  end;

  TFichaSuministroSimpleCombustible = class(TFichaLPD)
    QMax: NReal;   // Max Unidades / s
    precio: NREal; // USD / UNIDAD
    fd: NReal;
    TMR: NReal;
    indicePreciosPorCombustible: TFuenteAleatoria;
    bornePreciosPorCombustible: string;


    constructor Create(capa: integer; fecha: TFecha;
      periodicidad: TPeriodicidad; QMax,
      Precio: NReal;
      indicePreciosPorCombustible: TFuenteAleatoria;
      bornePreciosPorCombustible: string;
      fd, TMR: NReal);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function InfoAd_: string; override;
  end;

  TSuministroSimpleCombustible = class(TSuministroCombustible)
  public
    // Parámetros estaticos
    idxPrecio: TFuenteAleatoria;
    borne_idxPrecio: string;


    // variables
    pa: TFichaSuministroSimpleCombustible;
    kborne_idxPrecio: integer;
    Q: TDAOfNReal;
    QMaxOperativo: NReal;
    PrecioOperativo: NReal;
    NMaquinasDisponibles: integer;


    constructor Create(
       capa: integer; nombre: string; nacimiento, muerte: TFecha;
       lpdUnidades: TFichasLPD;
       lpd: TFichasLPD;
       NodoSumComb: TNodoCombustible);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure PubliVars; override;
    procedure PrepararMemoria(globs: TGlobs); override;

    class function DescClase: string; override;

    procedure RegistrarParametrosDinamicos; override;

    procedure SorteosDelPaso(sortear: boolean); virtual;
    procedure PrepararPaso_ps; virtual;

    procedure opt_nvers(var ivar, ivae, ires: integer); virtual;
    procedure opt_cargue(s: TSimplex); virtual;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); virtual;
    procedure opt_leerSolucion(s: TSimplex); virtual;


    procedure Free; override;




  end;

procedure AlInicio;
procedure ALFinal;

implementation



// Métodos de TSuministroCombustible

function TSuministroCombustible.referenciaAlNodo(NodoSumComb:
  TNodoCombustible): boolean;
begin
  Result := self.NodoCombustible =  NodoSumComb;
end;

constructor TSuministroCombustible.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText( f );
  f.rdReferencia( 'NodoCombustible', TCosa( NodoCombustible ), self );
end;

procedure TSuministroCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText( f );
  f.wrReferencia( 'NodoCombustible', Self.NodoCombustible );
end;




// Métodos de TFichaSUministroSimpleCombustible

constructor TFichaSuministroSimpleCombustible.Create(capa: integer;
  fecha: TFecha; periodicidad: TPeriodicidad; QMax,
  Precio: NReal;
  indicePreciosPorCombustible: TFuenteAleatoria;
  bornePreciosPorCombustible: string;
  fd, TMR: NReal);
begin
  inherited Create(capa, fecha, periodicidad);
  self.Qmax := Qmax;
  self.precio := precio;
  self.fd:= fd;
  self.TMR:= TMR;

  self.indicePreciosPorCombustible:= indicePreciosPorCombustible;
  self.bornePreciosPorCombustible:= bornePreciosPorCombustible;

end;

constructor TFichaSuministroSimpleCombustible.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText( f );
  f.IniciarLecturaRetrasada;
  f.rd('QMax', QMax);
  f.rd('Precio', Precio);
  f.rd( 'fd', fd );
  f.rd( 'TMR', TMR );
  f.rdReferencia('indicePreciosPorCombustible',
    TCosa(indicePreciosPorCombustible), self);
  f.rd('bornePreciosPorCombustible', bornePreciosPorCombustible);

  f.EjecutarLectura;
end;

procedure TFichaSuministroSimpleCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText( f );
  f.wr('QMax', QMax);
  f.wr('Precio', Precio);
  f.wr( 'fd', fd );
  f.wr( 'TMR', TMR );
  f.wrReferencia('indicePreciosPorCombustible', indicePreciosPorCombustible);
  f.wr('bornePreciosPorCombustible', bornePreciosPorCombustible);
end;

function TFichaSuministroSimpleCombustible.InfoAd_: string;
begin
  Result := 'QMax: ' + FloatToStr(QMax) + ', Precio:' + FloatToStr(Precio);
end;




//--------------------------
// Métodos de TSuministroSimpleCombustible
//==========================

constructor TSuministroSimpleCombustible.Create(capa: integer;
  nombre: string; nacimiento, muerte: TFecha;
  lpdUnidades: TFichasLPD;
  lpd: TFichasLPD;
  NodoSumComb: TNodoCombustible);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades);
  self.NodoCombustible := NodoSumComb;
  self.lpd:= lpd;
end;

constructor TSuministroSimpleCombustible.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.EjecutarLectura;
end;

procedure TSuministroSimpleCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
end;

procedure TSuministroSimpleCombustible.PubliVars;
begin
  inherited PubliVars;
end;


procedure TSuministroSimpleCombustible.PrepararMemoria(globs: TGlobs);
begin
  inherited PrepararMemoria(globs);
  setlength(Q, globs.NPostes);
  kborne_idxPrecio := idxPrecio.IdBorne(borne_idxPrecio);
end;


class function TSuministroSimpleCombustible.DescClase: string;
begin
  result:= rsSuministroSimpleCombustible;
end;

procedure TSuministroSimpleCombustible.RegistrarParametrosDinamicos;
begin
  inherited registrarParametrosDinamicos;
  lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pA, nil);
end;


procedure TSuministroSimpleCombustible.SorteosDelPaso(sortear: boolean);
begin
  if hayForzamientos or globs.ObligarDisponibilidad_1_ then
  begin
    NMaquinasDisponibles := paUnidades.nUnidades[0];   //supongo que nUnidades[0] tiene el total de maquinas de la central
    QMaxOperativo := pa.QMax * NMaquinasDisponibles ;
  end
  else if sortear then
  begin
    ActualizarProbabilidadesReparacionYRotura_(pa.fd, pa.TMR);
    NMaquinasDisponibles := Sorteos_RepRotUnidades;
    QMaxOperativo := pa.QMax * NMaquinasDisponibles ;
  end
  else
  begin
    NMaquinasDisponibles := paUnidades.nUnidades[0];            //si no hay que sortear deja todas las maquinas disponibles
    QMaxOperativo := pa.QMax * pa.fd * NMaquinasDisponibles; //pero afecta a la potencia disponible por la disponibilidad
  end;

end;

procedure TSuministroSimpleCombustible.PrepararPaso_ps;
begin
  PrecioOperativo:= pa.precio * self.idxPrecio.Bornera[ self.kborne_idxPrecio ];
end;

procedure TSuministroSimpleCombustible.opt_nvers(var ivar, ivae, ires: integer);
begin
  self.ivar := ivar;
  Inc(ivar, globs.npostes);
end;

procedure TSuministroSimpleCombustible.opt_cargue(s: TSimplex);
var
  iresnodo: integer;
  kposte: integer;
begin
  iresnodo := self.NodoCombustible.ires;
  for kposte := 0 to globs.npostes - 1 do
  begin
    s.pon_e(iresnodo, ivar + kposte, 1);

 // cargar funcion objetivo -costo
    s.pon_e(s.nf, ivar + kposte, -PrecioOperativo * globs.durpos[kposte]);
  end;


end;

procedure TSuministroSimpleCombustible.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  kposte: integer;
begin
  for kposte := 0 to globs.npostes - 1 do
    s.cota_sup_set(ivar + kposte, QMaxOperativo);
end;

procedure TSuministroSimpleCombustible.opt_leerSolucion(s: TSimplex);
var
  kposte: integer;
  Consumo: NREal;
begin
  Consumo:= 0;
  for kposte := 0 to globs.npostes - 1 do
  begin
    Q[kposte] := s.xval(ivar + kposte);
    Consumo:= Consumo + Q[kposte] * globs.DurPos[kposte];
  end;


  CostoDirectoDelPaso :=  Consumo * PrecioOperativo;
end;


procedure TSuministroSimpleCombustible.Free;
begin
  setlength(Q, 0);
  inherited Free;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TSuministroSimpleCombustible.ClassName,
    TSuministroSimpleCombustible);
  registrarClaseDeCosa(TFichaSuministroSimpleCombustible.ClassName,
    TFichaSuministroSimpleCombustible);

end;

procedure AlFinal;
begin
end;

end.
